/*
 * ws2bth.h
 */
#ifndef	_WS2BTH_H_
#define	_WS2BTH_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <windows.h>
#include <winnt.h>

/*
 * The Bluetooth device address stores the network address of a
 * Bluetooth–enabled device. It is used to identify a particular device
 * during operations such as connecting to, pairing with, or activating
 * the device.
 *
 * A Bluetooth–enabled device address is a unique, 48–bit address
 * containing the following three fields:
 *
 * * LAP field: lower part of the address containing 24 bits.
 * * UAP field: upper part of the address containing 8 bits.
 * * NAP field: non–significant part of the address containing 16 bits.
 *
 * The LAP and the UAP represent the significant address part (SAP) of
 * the Bluetooth device address.
 */
typedef ULONGLONG bt_addr, *pbt_addr, BT_ADDR, *PBT_ADDR;
typedef ULONG bt_cod, BT_COD, bt_lap, BT_LAP;

#define GET_NAP(bt_addr)	(((bt_addr) & 0xFFFF00000000) >> 32)
#define GET_SAP(bt_addr)	((bt_addr) & 0x0000FFFFFFFF)

#define	BLUETOOTH_MAX_NAME_SIZE		48	/* ?? */
#define	BTH_MAX_NAME_SIZE		48	/* ?? */

#define	BTH_SDP_VERSION			0	/* ?? */

#define	NS_BTH				16	/* ?? */

typedef struct _SOCKADDR_BTH {
	USHORT addressFamily;
	BT_ADDR btAddr;
	GUID serviceClassId;
	ULONG port;
} SOCKADDR_BTH, *PSOCKADDR_BTH;

typedef struct _BLUETOOTH_ADDRESS {
	union {
		BT_ADDR ullLong;
		BYTE rgBytes[6];
	} ;
} BLUETOOTH_ADDRESS;

typedef struct _BLUETOOTH_COD_PAIRS {
	ULONG ulCODMask;
	LPCWSTR pcszDescription;
} BLUETOOTH_COD_PAIRS;

typedef struct _BLUETOOTH_DEVICE_INFO {
	DWORD dwSize;
	BLUETOOTH_ADDRESS Address;
	ULONG ulClassofDevice;
	BOOL fConnected;
	BOOL fRemembered;
	BOOL fAuthenticated;
	SYSTEMTIME stLastSeen;
	SYSTEMTIME stLastUsed;
	WCHAR szName[BLUETOOTH_MAX_NAME_SIZE];
} BLUETOOTH_DEVICE_INFO, *PBLUETOOTH_DEVICE_INFO;

typedef struct {
	DWORD dwSize;
	BOOL fReturnAuthenticated;
	BOOL fReturnRemembered;
	BOOL fReturnUnknown;
	BOOL fReturnConnected;
	BOOL fIssueInquiry;
	UCHAR cTimeoutMultiplier;
	HANDLE hRadio;
} BLUETOOTH_DEVICE_SEARCH_PARAMS;

typedef struct {
	DWORD dwSize;
} BLUETOOTH_FIND_RADIO_PARAMS;

#if 0
typedef struct _BLUETOOTH_LOCAL_SERVICE_INFO {
	BOOL Enabled;
	BLUETOOTH_ADDRESS btAddr;
	WCHAR szName[BLUETOOTH_MAX_SERVICE_NAME_SIZE];
	WCHAR szDeviceString[BLUETOOTH_DEVICE_NAME_SIZE];
} BLUETOOTH_LOCAL_SERVICE_INFO;
#endif

#if 0
typedef struct {
	DWORD dwSize;
	BLUETOOTH_ADDRESS address;
	WCHAR szName[BLUETOOTH_MAX_NAME_SIZE];
	ULONG ulClassofDevice;
	USHORT lmpSubversion;
	USHORT manufacturer;
} BLUETOOTH_RADIO_INFO;
#endif


typedef BOOL PFN_DEVICE_CALLBACK(
		LPVOID pvParam,
		PBLUETOOTH_DEVICE_INFO pDevice);

typedef struct _BLUETOOTH_SELECT_DEVICE_PARAMS {
	DWORD dwSize;
	ULONG cNumOfClasses;
	BLUETOOTH_COD_PAIRS *prgClassOfDevices;
	LPWSTR pszInfo;
	HWND hwndParent;
	BOOL fForceAuthentication;
	BOOL fShowAuthenticated;
	BOOL fShowRemembered;
	BOOL fShowUnknown;
	BOOL fAddNewDeviceWizard;
	BOOL fSkipServicesPage;
	PFN_DEVICE_CALLBACK pfnDeviceCallback;
	LPVOID pvParam;
	DWORD cNumDevices;
	PBLUETOOTH_DEVICE_INFO pDevices;
} BLUETOOTH_SELECT_DEVICE_PARAMS;

typedef struct _BTH_DEVICE_INFO {
	ULONG flags;
	BT_ADDR address;
	BT_COD classOfDevice;
	CHAR name[BTH_MAX_NAME_SIZE];
} BTH_DEVICE_INFO, *PBTH_DEVICE_INFO;

typedef struct _BTH_HCI_EVENT_INFO {
	BT_ADDR bthAddress;
	UCHAR connectionType;
	UCHAR connected;
} BTH_HCI_EVENT_INFO, *PBTH_HCI_EVENT_INFO;

typedef struct _BTH_L2CAP_EVENT_INFO {
	BT_ADDR bthAddress;
	USHORT psm;
	UCHAR connected;
	UCHAR initiated;
} BTH_L2CAP_EVENT_INFO, *PBTH_L2CAP_EVENT_INFO;

typedef struct _BTH_QUERY_DEVICE {
	ULONG LAP;
	UCHAR length;
} BTH_QUERY_DEVICE, *PBTH_QUERY_DEVICE;

#if 0
typedef struct _BTH_QUERY_SERVICE {
	ULONG type;
	ULONG serviceHandle;
	SdpQueryUuid uuids[MAX_UUIDS_IN_QUERY];
	ULONG numRange;
	SdpAttributeRange pRange[1];
} BTH_QUERY_SERVICE, *PBTH_QUERY_SERVICE;
#endif

typedef struct _BTH_RADIO_IN_RANGE {
	BTH_DEVICE_INFO deviceInfo;
	ULONG previousDeviceFlags;
} BTH_RADIO_IN_RANGE, *PBTH_RADIO_IN_RANGE;

typedef struct _BTH_SET_SERVICE {
	PULONG pSdpVersion;
	HANDLE *pRecordHandle;
	ULONG fCodService;
	ULONG Reserved[5];
	ULONG ulRecordLength;
	UCHAR pRecord[1];
} BTH_SET_SERVICE, *PBTH_SET_SERVICE;

typedef struct _BTHNS_SETBLOB {
	ULONG *pSdpVersion;	/* Mysteriously missing on MSDN */
	ULONG* pRecordHandle;
	ULONG fSecurity;
	ULONG fOptions;
	ULONG ulRecordLength;
	UCHAR pRecord[1];
} BTHNS_SETBLOB, *PBTHNS_SETBLOB;

#ifdef __cplusplus
}
#endif
#endif /* _WS2BTH_H_ */
